@@ -1,4 +1,4 @@
-
+import warnings
 import numpy as np
 
 
@@ -13,7 +13,7 @@ class Node(object):
         :param inbound_edges:list,collections of all input edges
         '''
 
-        self.inbounds=[]
+        self.inbounds = []
 
         self.outbound_layers=[] if outbound_layers is None else list(outbound_layers)
 
@@ -96,10 +96,18 @@ class Node(object):
 
 
 class Constant(Node):
-    def __init__(self,value,name='constant'):
-        Node.__init__(self)
-        self.output_tensor=np.array(value)
-        self.name=name
+    def __init__(self, output_tensor, name='constant'):
+        super(Constant, self).__init__(output_tensor=np.array(output_tensor))
+        self.name = name
+        self._locked = True
+
+
+
+    def __setattr__(self, key, value):
+        if self.__dict__.get("_locked") and key == 'output_tensor':
+            warnings.warn('Can not change Constant value!')
+        else:
+            self.__dict__[key] = value
 
 
 
@@ -182,9 +190,10 @@ class Layer(object):
 
         self.variables=[] if variables is None else list(variables)
         #record how many other layers require this layer
-        self.counts=0
-        self.require_grads=True
-        self.grads=None
+        self.counts = 0
+        self.require_grads = True
+        self.grads = None
+        self.name = None
 
 
     def __call__(self, inbound):
@@ -251,6 +260,16 @@ class Layer(object):
         return Multiply()([self, other])
 
 
+
+
+    def params_count(self):
+        total_params = 0
+        for variable in self.variables:
+            total_params += variable.output_tensor.size
+        return total_params
+
+
+
     def __neg__(self):
         return Negative()(self)
 
@@ -354,14 +373,18 @@ class Add(Operation):
 
     def __call__(self, inbounds):
         super(Add,self).__call__(inbounds)
-        x,y=inbounds
-        shape_a=x.output_tensor.shape
-        shape_b=y.output_tensor.shape
-        assert len(shape_a)==len(shape_b)
-        output_shape=tuple()
-        for a,b in zip(shape_a[1:],shape_b[1:]):
-            output_shape+=(max(a,b),)
-        output_shape=(None,)+output_shape
+        x, y = inbounds
+        shape_a = x.output_shape
+        shape_b = y.output_shape
+        unequal_dims = 0
+        output_shape = tuple()
+        for a, b in zip(shape_a[1:], shape_b[1:]):
+            if a != b:
+                unequal_dims += 1
+            if unequal_dims > 1:
+                raise ValueError('Can not add two tensors with shape: %s, %s' % (shape_a, shape_b))
+            output_shape += (max(a, b),)
+        output_shape = (None,) + output_shape
         self.output_shape=output_shape
         if x.output_tensor is not None and y.output_tensor is not None:
             self.output_tensor=x.output_tensor+y.output_tensor
@@ -419,7 +442,7 @@ class Negative(Operation):
     def __call__(self, inbound):
         super(Negative,self).__call__([inbound])
         x=inbound
-        self.output_shape=x.output_shape
+        self.output_shape = x.output_shape
         if x.output_tensor is not None:
             self.output_tensor=-x.output_tensor
             if x.require_grads:
@@ -452,10 +475,9 @@ class Multiply(Operation):
     def __call__(self, inbounds):
         super(Multiply,self).__call__(inbounds)
         x,y=inbounds
-        shape_a=x.output_tensor.shape
-        shape_b=y.output_tensor.shape
-        assert shape_a==shape_b
-        self.output_shape=shape_a
+        shape_a = x.output_shape
+        shape_b = y.output_shape
+        self.output_shape = shape_a
         if x.output_tensor is not None and y.output_tensor is not None:
             self.output_tensor=x.output_tensor*y.output_tensor
             if x.require_grads:
