@@ -7,10 +7,42 @@ import os
 import pickle
 
 
+class BaseModel():
+    def __init__(self):
+        pass
 
 
-class Sequential():
+
+    def format_time(self, second_time):
+        if second_time < 1:
+            ms = second_time * 1000
+            if ms < 1:
+                us = second_time * 1000
+                return '%dus' % us
+            else:
+                return '%dms' % ms
+        second_time = round(second_time)
+        if second_time > 3600:
+            # hours
+            h = second_time // 3600
+            second_time = second_time % 3600
+            # minutes
+            m = second_time // 60
+            second_time = second_time % 60
+            return '%dh%dm%ds' % (h, m, second_time)
+        elif second_time > 60:
+            m = second_time // 60
+            second_time = second_time % 60
+            return '%dm%ds' % (m, second_time)
+        else:
+            return '%ds' % second_time
+
+
+
+
+class Sequential(BaseModel):
     def __init__(self,layers=None):
+        super(Sequential, self).__init__()
         self.layers=[] if layers is None else layers
         self.train_loss=[]
         self.train_acc=[]
@@ -60,7 +92,7 @@ class Sequential():
 
 
         for epoch in range(epochs):
-            mini_batches=get_batches(train_X,train_Y,batch_size,epoch,shuffle)
+            mini_batches = get_batches(train_X,train_Y,batch_size,epoch,shuffle)
             batch_nums=len(mini_batches)
             training_size=train_X.shape[0]
             batch_count=0
@@ -110,12 +142,12 @@ class Sequential():
 
 
                 trained_process_bar_nums=batch_count*self.process_bar_nums//batch_nums
-                process_bar=self.process_bar_trained*trained_process_bar_nums+'>'+self.process_bar_untrain*(self.process_bar_nums-trained_process_bar_nums-1)
+                process_bar = self.process_bar_trained*trained_process_bar_nums+'>'+self.process_bar_untrain*(self.process_bar_nums-trained_process_bar_nums-1)
                 if validation_data is not None:
                     print(
-                        '\r{:d}/{:d} [{}] -{:.0f}s -{:.0f}ms/batch -batch_loss: {:.4f} -batch_acc: {:.4f} -val_loss: {:.4f} -val_acc: {:.4f}'.format(trained_nums, training_size, process_bar, gap, gap*1000 / batch_count,batch_loss, batch_acc, valid_loss, valid_acc), end='')
+                        '\r{:d}/{:d} [{}] -{} -{}/batch -batch_loss: {:.4f} -batch_acc: {:.4f} -val_loss: {:.4f} -val_acc: {:.4f}'.format(trained_nums, training_size, process_bar, self.format_time(gap), self.format_time(gap / batch_count), batch_loss, batch_acc, valid_loss, valid_acc), end='')
                 else:
-                    print('\r{:d}/{:d} [{}] -{:.0f}s -{:.0f}ms/batch -batch_loss: {:.4f} -batch_acc: {:.4f} '.format( trained_nums, training_size, process_bar, gap, gap * 1000 / batch_count,batch_loss, batch_acc), end='')
+                    print('\r{:d}/{:d} [{}] -{} -{}/batch -batch_loss: {:.4f} -batch_acc: {:.4f} '.format( trained_nums, training_size, process_bar, self.format_time(gap), self.format_time(gap / batch_count), batch_loss, batch_acc), end='')
             print()
 
 
@@ -269,8 +301,9 @@ class Sequential():
 
 
 
-class Model():
+class Model(BaseModel):
     def __init__(self, inputs=None,outputs=None):
+        super(BaseModel, self).__init__()
         self.inputs=inputs
         self.outputs=outputs
         self.train_loss = []
@@ -283,7 +316,6 @@ class Model():
 
 
 
-
     def topological_sort(self,input_layers,mode='forward'):
         """
         Sort generic nodes in topological order using Kahn's Algorithm.
@@ -408,7 +440,10 @@ class Model():
                 self.train_loss.append(batch_loss)
                 self.train_acc.append(batch_acc)
                 if validation_data is not None:
-                    valid_acc, valid_loss = self.evaluate(valid_X, valid_Y,batch_size=batch_size)
+                    if valid_X.shape[0] > batch_size:
+                        valid_acc, valid_loss = self.evaluate(valid_X, valid_Y, batch_size=batch_size)
+                    else:
+                        valid_acc, valid_loss = self.evaluate(valid_X, valid_Y)
                     self.valid_loss.append(valid_loss)
                     self.valid_acc.append(valid_acc)
 
@@ -425,9 +460,9 @@ class Model():
                 trained_process_bar_nums = batch_count * self.process_bar_nums // batch_nums
                 process_bar = self.process_bar_trained * trained_process_bar_nums + '>' + self.process_bar_untrain * ( self.process_bar_nums - trained_process_bar_nums - 1)
                 if validation_data is not None:
-                    print( '\r{:d}/{:d} [{}] -{:.0f}s -{:.0f}ms/batch -batch_loss: {:.4f} -batch_acc: {:.4f} -val_loss: {:.4f} -val_acc: {:.4f}'.format(trained_nums,training_size,process_bar, gap, gap*1000/batch_count,batch_loss, batch_acc, valid_loss, valid_acc), end='')
+                    print( '\r{:d}/{:d} [{}] -{} -{}/batch -batch_loss: {:.4f} -batch_acc: {:.4f} -val_loss: {:.4f} -val_acc: {:.4f}'.format(trained_nums,training_size,process_bar, self.format_time(gap), self.format_time(gap / batch_count),batch_loss, batch_acc, valid_loss, valid_acc), end='')
                 else:
-                    print('\r{:d}/{:d} [{}] -{:.0f}s -{:.0f}ms/batch -batch_loss: {:.4f} -batch_acc: {:.4f} '.format(trained_nums,training_size,process_bar, gap,gap*1000/batch_count, batch_loss, batch_acc), end='')
+                    print('\r{:d}/{:d} [{}] -{} -{}/batch -batch_loss: {:.4f} -batch_acc: {:.4f} '.format(trained_nums,training_size,process_bar, self.format_time(gap), self.format_time(gap / batch_count), batch_loss, batch_acc), end='')
             print()
 
 
